# frozen_string_literal: true

module Jekyll
  module Filters
    # YAML to SCSS
    module YamlToScss
      # @see {https://sass-lang.com/documentation/values/strings#unquoted}
      CSS_IDENTIFIER_SHORT = /\A-?\w+\z/
      CSS_IDENTIFIER_LONG  = /\A--[\w0-9]+\z/
      SCSS_FUNCTION = /\A[0-9a-z]+\(/

      # Converts a YAML object into SCSS.  Only supports one level
      # values.
      #
      # Optionally give it an schema to filter out keys.
      #
      # @param :yaml [Hash,Jekyll::Drops::DocumentDrop]
      # @param :schema [Hash]
      # @return [String]
      def yaml_to_scss(yaml, schema = yaml)
        yaml = yaml.to_h if yaml.is_a? Jekyll::Drops::DocumentDrop

        unless yaml.is_a? Hash
          raise Liquid::ArgumentError, "#{yaml.inspect} needs to be a Hash" if @context.strict_filters

          return yaml
        end

        yaml.slice(*schema.keys).map do |key, value|
          next if value.nil?

          key = key_to_scss key

          case value
          when Array
            values = value.compact.map(&:to_s).reject(&:empty?).map do |v|
              scss_expression?(v) ? v : v.dump
            end

            "$#{key}: (#{values.join(',')},);"
          when Hash
            values =
              value.map do |k, v|
                next if v.nil?
                next if v.to_s.empty?

                v = v.dump unless scss_expression? v

                "\"#{key_to_scss k}\": #{v}"
              end

            "$#{key}: (#{values.join(',')},);"
          else
            next if value.to_s.empty?
            value = value.dump unless scss_expression? value

            "$#{key}: #{value};"
          end
        end.join("\n")
      end

      private

      # Dasherize key
      #
      # @param :key [String]
      # @return [String]
      def key_to_scss(key)
        key.to_s.tr '_', '-'
      end

      # Detects if value is a SCSS/SASS expression
      #
      # @param :value [Any]
      # @return [Bool]
      def scss_expression?(value)
        return true unless value.is_a? String
        # Quoted?
        return false if value.start_with? '"'
        return false if value.start_with? '\''
        # Array/hash?
        return true if value.start_with? '('
        # Variable?
        return true if value.start_with? '$'
        # RGB color?
        return true if value.start_with? '#'
        # Negative number or CSS identifier?
        return true if value.start_with? '-'

        # Number?
        (0..9).each do |i|
          return true if value.start_with? i.to_s
        end

        # CSS identifier?
        return true if CSS_IDENTIFIER_SHORT =~ value
        return true if CSS_IDENTIFIER_LONG =~ value
        return true if SCSS_FUNCTION =~ value

        false
      end
    end
  end
end

Liquid::Template.register_filter(Jekyll::Filters::YamlToScss)
